#/bin/bash

USAGE="Usage: `basename $0` conf tmux [spire_dir] [log_dir]
    conf        - configuration dir
    tmux        - (0/1) whether to run process in a tmux session
    spire_dir   - (optional, default ~/spire-src-1.3) directory of spire src code
    log_dir     - (optional, default ~/spire-logs) directory where output is logged, will be created"

MIN_ARGS=2
MAX_ARGS=4

if [ $# -lt $MIN_ARGS -o $# -gt $MAX_ARGS ]
then
    echo "$USAGE"
    exit
fi

CONF_DIR=${1%/}
WINDOW=$2
SPIRE_DIR=${3-"~/spire-src-1.3"}
FAULTY_SPIRE_DIR=${3-"~/faulty-spire"}
LOG_DIR=${4-"~/spire-logs"}

PRIME_DIR="$SPIRE_DIR/prime/bin"
FAULTY_PRIME_DIR="$FAULTY_SPIRE_DIR/prime/bin"
SM_FILE="$CONF_DIR/sm.txt"

PRIME_CMD_BASE="cd $PRIME_DIR; ./prime"
FAULTY_PRIME_CMD_BASE="cd $FAULTY_PRIME_DIR; ./prime"


# Start Prime
i=0
while read -r -a line
do
    mach_name=${line[0]}
    if [ $i -eq 4 ]
    then
    cmd="$FAULTY_PRIME_CMD_BASE -i $((i+1))"
    else
    cmd="$PRIME_CMD_BASE -i $((i+1))"
    fi 

    outfile="$LOG_DIR/prime$(($i+1)).out"

    echo "Starting $mach_name > \`$cmd\`"

    # Make log dir
    ssh -n $mach_name "mkdir -p $LOG_DIR";

    if [ $WINDOW -eq 0 ]
    then
        ssh -n $mach_name "$cmd > $outfile 2>&1 &" > /dev/null 2>&1 &
    else
	    tmux new-session -d -s "$mach_name prime" "ssh -n $mach_name '$cmd 2>&1 | tee $outfile'" 
    fi
    i=$(($i+1))
done < $SM_FILE
